from django.test import TestCase
from django.utils import timezone
from django.db.models.functions import Now

from evotevotings.processes.models import ElectoralProcess

from evotevotings.processes.tests.utils import ProcessesTestUtils
from evoteusers.tests.utils import UserTestUtils


class ElectoralProcessManagerTest(ProcessesTestUtils, UserTestUtils, TestCase):
    """
    Test Managers of Electoral Process
    """

    def test_open(self):
        self.create_electoral_process(state=ElectoralProcess.State.OPEN)
        self.create_electoral_process(state=ElectoralProcess.State.NEW)
        valid_qs = ElectoralProcess.objects.filter(
            state=ElectoralProcess.State.OPEN)
        expected_qs = ElectoralProcess.objects.open()
        self.assertQuerysetEqual(valid_qs, map(repr, expected_qs))

    def test_published(self):
        self.create_electoral_process(publication_date=timezone.now())
        self.create_electoral_process(publication_date=timezone.now())
        self.create_electoral_process(
            publication_date=timezone.now(), state=ElectoralProcess.State.OPEN)
        valid_qs = ElectoralProcess.objects.filter(
            publication_date__lte=Now(),
            state=ElectoralProcess.State.OPEN)
        expected_qs = ElectoralProcess.objects.published()
        self.assertQuerysetEqual(valid_qs, map(repr, expected_qs))

    def test_closed(self):
        self.create_electoral_process(state=ElectoralProcess.State.OPEN)
        self.create_electoral_process(state=ElectoralProcess.State.CLOSED)
        self.create_electoral_process(state=ElectoralProcess.State.ARCHIVED)
        expected_states = [
            ElectoralProcess.State.CLOSED, ElectoralProcess.State.ARCHIVED]
        valid_qs = ElectoralProcess.objects.filter(state__in=expected_states)
        expected_qs = ElectoralProcess.objects.closed()
        self.assertEqual(expected_qs.count(), 2)
        self.assertQuerysetEqual(valid_qs, map(repr, expected_qs))

    def test_public(self):
        self.create_electoral_process(state=ElectoralProcess.State.OPEN)
        self.create_electoral_process(state=ElectoralProcess.State.NEW)
        self.create_electoral_process(state=ElectoralProcess.State.CLOSED)
        self.create_electoral_process(state=ElectoralProcess.State.ARCHIVED)
        expected_states = [
            ElectoralProcess.State.NEW, ElectoralProcess.State.OPEN,
            ElectoralProcess.State.CLOSED]
        valid_qs = ElectoralProcess.objects.filter(state__in=expected_states)
        expected_qs = ElectoralProcess.objects.public()
        self.assertEqual(expected_qs.count(), 3)
        self.assertQuerysetEqual(valid_qs, map(repr, expected_qs))
